@@ -7,39 +7,49 @@
 # Copyright (c) 2015 Florian Bruhin <me@the-compiler.org>
 # Copyright (c) 2016-2017 Derek Gustafson <degustaf@gmail.com>
 # Copyright (c) 2017 Calen Pennington <calen.pennington@gmail.com>
+# Copyright (c) 2018-2019 Nick Drozd <nicholasdrozd@gmail.com>
 # Copyright (c) 2018-2019 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2018 Ville Skyttä <ville.skytta@iki.fi>
 # Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
-# Copyright (c) 2018 Nick Drozd <nicholasdrozd@gmail.com>
 # Copyright (c) 2018 Daniel Colascione <dancol@dancol.org>
 # Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
 # Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 doranid <ddandd@gmail.com>
+# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 # Copyright (c) 2021 Andrew Haigh <hello@nelf.in>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/LICENSE
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 
 """This module contains base classes and functions for the nodes and some
 inference utils.
 """
 
-import builtins
 import collections
 
 from astroid import context as contextmod
-from astroid import exceptions, util
+from astroid import decorators, util
+from astroid.const import PY310_PLUS
+from astroid.exceptions import (
+    AstroidTypeError,
+    AttributeInferenceError,
+    InferenceError,
+    NameInferenceError,
+)
 
 objectmodel = util.lazy_import("interpreter.objectmodel")
 helpers = util.lazy_import("helpers")
-BUILTINS = builtins.__name__
 manager = util.lazy_import("manager")
-MANAGER = manager.AstroidManager()
+
 
 # TODO: check if needs special treatment
-BUILTINS = "builtins"
 BOOL_SPECIAL_METHOD = "__bool__"
+BUILTINS = "builtins"  # TODO Remove in 2.8
+
+PROPERTIES = {"builtins.property", "abc.abstractproperty"}
+if PY310_PLUS:
+    PROPERTIES.add("enum.property")
 
-PROPERTIES = {BUILTINS + ".property", "abc.abstractproperty"}
 # List of possible property names. We use this list in order
 # to see if a method is a property or not. This should be
 # pretty reliable and fast, the alternative being to check each
@@ -59,6 +69,7 @@ POSSIBLE_PROPERTIES = {
     "LazyProperty",
     "lazy",
     "cache_readonly",
+    "DynamicClassAttribute",
 }
 
 
@@ -84,7 +95,7 @@ def _is_property(meth, context=None):
                 if base_class.__class__.__name__ != "Name":
                     continue
                 module, _ = base_class.lookup(base_class.name)
-                if module.name == BUILTINS and base_class.name == "property":
+                if module.name == "builtins" and base_class.name == "property":
                     return True
 
     return False
@@ -133,16 +144,16 @@ def _infer_stmts(stmts, context, frame=None):
             continue
         context.lookupname = stmt._infer_name(frame, name)
         try:
-            for inferred in stmt.infer(context=context):
-                yield inferred
+            for inf in stmt.infer(context=context):
+                yield inf
                 inferred = True
-        except exceptions.NameInferenceError:
+        except NameInferenceError:
             continue
-        except exceptions.InferenceError:
+        except InferenceError:
             yield util.Uninferable
             inferred = True
     if not inferred:
-        raise exceptions.InferenceError(
+        raise InferenceError(
             "Inference failed for all members of {stmts!r}.",
             stmts=stmts,
             frame=frame,
@@ -161,10 +172,12 @@ def _infer_method_result_truth(instance, method_name, context):
             for value in meth.infer_call_result(instance, context=context):
                 if value is util.Uninferable:
                     return value
-
-                inferred = next(value.infer(context=context))
+                try:
+                    inferred = next(value.infer(context=context))
+                except StopIteration as e:
+                    raise InferenceError(context=context) from e
                 return inferred.bool_value()
-        except exceptions.InferenceError:
+        except InferenceError:
             pass
     return util.Uninferable
 
@@ -180,7 +193,7 @@ class BaseInstance(Proxy):
     def getattr(self, name, context=None, lookupclass=True):
         try:
             values = self._proxied.instance_attr(name, context)
-        except exceptions.AttributeInferenceError as exc:
+        except AttributeInferenceError as exc:
             if self.special_attributes and name in self.special_attributes:
                 return [self.special_attributes.lookup(name)]
 
@@ -189,7 +202,7 @@ class BaseInstance(Proxy):
                 # unless they are explicitly defined.
                 return self._proxied.getattr(name, context, class_context=False)
 
-            raise exceptions.AttributeInferenceError(
+            raise AttributeInferenceError(
                 target=self, attribute=name, context=context
             ) from exc
         # since we've no context information, return matching class members as
@@ -199,7 +212,7 @@ class BaseInstance(Proxy):
                 return values + self._proxied.getattr(
                     name, context, class_context=False
                 )
-            except exceptions.AttributeInferenceError:
+            except AttributeInferenceError:
                 pass
         return values
 
@@ -211,7 +224,7 @@ class BaseInstance(Proxy):
             context.lookupname = name
             # avoid recursively inferring the same attr on the same class
             if context.push(self._proxied):
-                raise exceptions.InferenceError(
+                raise InferenceError(
                     message="Cannot infer the same attribute again",
                     node=self,
                     context=context,
@@ -222,7 +235,7 @@ class BaseInstance(Proxy):
             yield from _infer_stmts(
                 self._wrap_attr(get_attr, context), context, frame=self
             )
-        except exceptions.AttributeInferenceError:
+        except AttributeInferenceError:
             try:
                 # fallback to class.igetattr since it has some logic to handle
                 # descriptors
@@ -231,8 +244,8 @@ class BaseInstance(Proxy):
                     raise
                 attrs = self._proxied.igetattr(name, context, class_context=False)
                 yield from self._wrap_attr(attrs, context)
-            except exceptions.AttributeInferenceError as error:
-                raise exceptions.InferenceError(**vars(error)) from error
+            except AttributeInferenceError as error:
+                raise InferenceError(**vars(error)) from error
 
     def _wrap_attr(self, attrs, context=None):
         """wrap bound methods of attrs in a InstanceMethod proxies"""
@@ -261,7 +274,7 @@ class BaseInstance(Proxy):
                 inferred = True
                 yield res
         if not inferred:
-            raise exceptions.InferenceError(node=self, caller=caller, context=context)
+            raise InferenceError(node=self, caller=caller, context=context)
 
 
 class Instance(BaseInstance):
@@ -282,7 +295,7 @@ class Instance(BaseInstance):
         try:
             self._proxied.getattr("__call__", class_context=False)
             return True
-        except exceptions.AttributeInferenceError:
+        except AttributeInferenceError:
             return False
 
     def pytype(self):
@@ -310,11 +323,11 @@ class Instance(BaseInstance):
 
         try:
             result = _infer_method_result_truth(self, BOOL_SPECIAL_METHOD, context)
-        except (exceptions.InferenceError, exceptions.AttributeInferenceError):
+        except (InferenceError, AttributeInferenceError):
             # Fallback to __len__.
             try:
                 result = _infer_method_result_truth(self, "__len__", context)
-            except (exceptions.AttributeInferenceError, exceptions.InferenceError):
+            except (AttributeInferenceError, InferenceError):
                 return True
         return result
 
@@ -327,16 +340,16 @@ class Instance(BaseInstance):
         new_context.callcontext = contextmod.CallContext(args=[index])
         method = next(self.igetattr("__getitem__", context=context), None)
         if not isinstance(method, BoundMethod):
-            raise exceptions.InferenceError(
+            raise InferenceError(
                 "Could not find __getitem__ for {node!r}.", node=self, context=context
             )
         if len(method.args.arguments) != 2:  # (self, index)
-            raise exceptions.AstroidTypeError(
+            raise AstroidTypeError(
                 "__getitem__ for {node!r} does not have correct signature",
                 node=self,
                 context=context,
             )
-        return next(method.infer_call_result(self, new_context))
+        return next(method.infer_call_result(self, new_context), None)
 
 
 class UnboundMethod(Proxy):
@@ -382,7 +395,7 @@ class UnboundMethod(Proxy):
         # instance of the class given as first argument.
         if (
             self._proxied.name == "__new__"
-            and self._proxied.parent.frame().qname() == "%s.object" % BUILTINS
+            and self._proxied.parent.frame().qname() == "builtins.object"
         ):
             if caller.args:
                 node_context = context.extra_context.get(caller.args[0])
@@ -423,19 +436,25 @@ class BoundMethod(UnboundMethod):
         needs to be a tuple of classes
         """
         # pylint: disable=import-outside-toplevel; circular import
-        from astroid import node_classes
+        from astroid.nodes import node_classes
 
         # Verify the metaclass
-        mcs = next(caller.args[0].infer(context=context))
+        try:
+            mcs = next(caller.args[0].infer(context=context))
+        except StopIteration as e:
+            raise InferenceError(context=context) from e
         if mcs.__class__.__name__ != "ClassDef":
             # Not a valid first argument.
             return None
-        if not mcs.is_subtype_of("%s.type" % BUILTINS):
+        if not mcs.is_subtype_of("builtins.type"):
             # Not a valid metaclass.
             return None
 
         # Verify the name
-        name = next(caller.args[1].infer(context=context))
+        try:
+            name = next(caller.args[1].infer(context=context))
+        except StopIteration as e:
+            raise InferenceError(context=context) from e
         if name.__class__.__name__ != "Const":
             # Not a valid name, needs to be a const.
             return None
@@ -444,24 +463,39 @@ class BoundMethod(UnboundMethod):
             return None
 
         # Verify the bases
-        bases = next(caller.args[2].infer(context=context))
+        try:
+            bases = next(caller.args[2].infer(context=context))
+        except StopIteration as e:
+            raise InferenceError(context=context) from e
         if bases.__class__.__name__ != "Tuple":
             # Needs to be a tuple.
             return None
-        inferred_bases = [next(elt.infer(context=context)) for elt in bases.elts]
+        try:
+            inferred_bases = [next(elt.infer(context=context)) for elt in bases.elts]
+        except StopIteration as e:
+            raise InferenceError(context=context) from e
         if any(base.__class__.__name__ != "ClassDef" for base in inferred_bases):
             # All the bases needs to be Classes
             return None
 
         # Verify the attributes.
-        attrs = next(caller.args[3].infer(context=context))
+        try:
+            attrs = next(caller.args[3].infer(context=context))
+        except StopIteration as e:
+            raise InferenceError(context=context) from e
         if attrs.__class__.__name__ != "Dict":
             # Needs to be a dictionary.
             return None
         cls_locals = collections.defaultdict(list)
         for key, value in attrs.items:
-            key = next(key.infer(context=context))
-            value = next(value.infer(context=context))
+            try:
+                key = next(key.infer(context=context))
+            except StopIteration as e:
+                raise InferenceError(context=context) from e
+            try:
+                value = next(value.infer(context=context))
+            except StopIteration as e:
+                raise InferenceError(context=context) from e
             # Ignore non string keys
             if key.__class__.__name__ == "Const" and isinstance(key.value, str):
                 cls_locals[key.value].append(value)
@@ -510,18 +544,22 @@ class Generator(BaseInstance):
     Proxied class is set once for all in raw_building.
     """
 
-    # pylint: disable=unnecessary-lambda
-    special_attributes = util.lazy_descriptor(lambda: objectmodel.GeneratorModel())
+    special_attributes = util.lazy_descriptor(objectmodel.GeneratorModel)
 
-    # pylint: disable=super-init-not-called
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, generator_initial_context=None):
+        super().__init__()
         self.parent = parent
+        self._call_context = contextmod.copy_context(generator_initial_context)
+
+    @decorators.cached
+    def infer_yield_types(self):
+        yield from self.parent.infer_yield_result(self._call_context)
 
     def callable(self):
         return False
 
     def pytype(self):
-        return "%s.generator" % BUILTINS
+        return "builtins.generator"
 
     def display_type(self):
         return "Generator"
@@ -535,14 +573,14 @@ class Generator(BaseInstance):
         )
 
     def __str__(self):
-        return "Generator(%s)" % (self._proxied.name)
+        return "Generator(%s)" % self._proxied.name
 
 
 class AsyncGenerator(Generator):
     """Special node representing an async generator"""
 
     def pytype(self):
-        return "%s.async_generator" % BUILTINS
+        return "builtins.async_generator"
 
     def display_type(self):
         return "AsyncGenerator"
@@ -553,4 +591,4 @@ class AsyncGenerator(Generator):
         )
 
     def __str__(self):
-        return "AsyncGenerator(%s)" % (self._proxied.name)
+        return "AsyncGenerator(%s)" % self._proxied.name
