@@ -12,34 +12,27 @@
 # Copyright (c) 2020 David Gilman <davidgilman1@gmail.com>
 # Copyright (c) 2020 Ram Rachum <ram@rachum.com>
 # Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Nick Drozd <nicholasdrozd@gmail.com>
 # Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/LICENSE
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 
 """Astroid hooks for various builtins."""
 
 from functools import partial
 
-from astroid import (
-    arguments,
-    helpers,
-    inference_tip,
-    nodes,
-    objects,
-    scoped_nodes,
-    util,
-)
+from astroid import arguments, helpers, inference_tip, nodes, objects, util
 from astroid.builder import AstroidBuilder
 from astroid.exceptions import (
     AstroidTypeError,
     AttributeInferenceError,
     InferenceError,
     MroError,
-    NameInferenceError,
     UseInferenceDefault,
 )
 from astroid.manager import AstroidManager
+from astroid.nodes import scoped_nodes
 
 OBJECT_DUNDER_NEW = "object.__new__"
 
@@ -200,7 +193,9 @@ def register_builtin_transform(transform, builtin_name):
 
             if result.lineno is None:
                 result.lineno = node.lineno
-            if result.col_offset is None:
+            # Can be a 'Module' see https://github.com/PyCQA/pylint/issues/4671
+            # We don't have a regression test on this one: tread carefully
+            if hasattr(result, "col_offset") and result.col_offset is None:
                 result.col_offset = node.col_offset
         return iter([result])
 
@@ -329,7 +324,7 @@ def _get_elts(arg, context):
 
     try:
         inferred = next(arg.infer(context))
-    except (InferenceError, NameInferenceError, StopIteration) as exc:
+    except (InferenceError, StopIteration) as exc:
         raise UseInferenceDefault from exc
     if isinstance(inferred, nodes.Dict):
         items = inferred.items
@@ -663,7 +658,7 @@ def infer_issubclass(callnode, context=None):
     """Infer issubclass() calls
 
     :param nodes.Call callnode: an `issubclass` call
-    :param InferenceContext: the context for the inference
+    :param InferenceContext context: the context for the inference
     :rtype nodes.Const: Boolean Const value of the `issubclass` call
     :raises UseInferenceDefault: If the node cannot be inferred
     """
@@ -708,7 +703,7 @@ def infer_isinstance(callnode, context=None):
     """Infer isinstance calls
 
     :param nodes.Call callnode: an isinstance call
-    :param InferenceContext: context for call
+    :param InferenceContext context: context for call
         (currently unused but is a common interface for inference)
     :rtype nodes.Const: Boolean Const value of isinstance call
 
@@ -845,7 +840,7 @@ def infer_dict_fromkeys(node, context=None):
     """Infer dict.fromkeys
 
     :param nodes.Call node: dict.fromkeys() call to infer
-    :param context.InferenceContext: node context
+    :param context.InferenceContext context: node context
     :rtype nodes.Dict:
         a Dictionary containing the values that astroid was able to infer.
         In case the inference failed for any reason, an empty dictionary
