@@ -9,7 +9,7 @@
 # Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
 # Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/LICENSE
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 """
 Data object model, as per https://docs.python.org/3/reference/datamodel.html.
 
@@ -38,9 +38,11 @@ from typing import Optional
 
 import astroid
 from astroid import context as contextmod
-from astroid import exceptions, node_classes, util
+from astroid import util
+from astroid.exceptions import AttributeInferenceError, InferenceError, NoDefault
+from astroid.manager import AstroidManager
+from astroid.nodes import node_classes
 
-# Prevents circular imports
 objects = util.lazy_import("objects")
 
 
@@ -120,12 +122,12 @@ class ObjectModel:
 
         if name in self.attributes():
             return getattr(self, IMPL_PREFIX + name)
-        raise exceptions.AttributeInferenceError(target=self._instance, attribute=name)
+        raise AttributeInferenceError(target=self._instance, attribute=name)
 
 
 class ModuleModel(ObjectModel):
     def _builtins(self):
-        builtins_ast_module = astroid.MANAGER.builtins_module
+        builtins_ast_module = AstroidManager().builtins_module
         return builtins_ast_module.special_attributes.lookup("__dict__")
 
     @property
@@ -135,9 +137,7 @@ class ModuleModel(ObjectModel):
     @property
     def attr___path__(self):
         if not self._instance.package:
-            raise exceptions.AttributeInferenceError(
-                target=self._instance, attribute="__path__"
-            )
+            raise AttributeInferenceError(target=self._instance, attribute="__path__")
 
         path_objs = [
             node_classes.Const(
@@ -268,7 +268,7 @@ class FunctionModel(ObjectModel):
             for arg in args.kwonlyargs:
                 try:
                     default = args.default_value(arg.name)
-                except exceptions.NoDefault:
+                except NoDefault:
                     continue
 
                 name = node_classes.Const(arg.name, parent=parent)
@@ -302,17 +302,20 @@ class FunctionModel(ObjectModel):
 
             def infer_call_result(self, caller, context=None):
                 if len(caller.args) > 2 or len(caller.args) < 1:
-                    raise exceptions.InferenceError(
+                    raise InferenceError(
                         "Invalid arguments for descriptor binding",
                         target=self,
                         context=context,
                     )
 
                 context = contextmod.copy_context(context)
-                cls = next(caller.args[0].infer(context=context))
+                try:
+                    cls = next(caller.args[0].infer(context=context))
+                except StopIteration as e:
+                    raise InferenceError(context=context, node=caller.args[0]) from e
 
                 if cls is astroid.Uninferable:
-                    raise exceptions.InferenceError(
+                    raise InferenceError(
                         "Invalid class inferred", target=self, context=context
                     )
 
@@ -417,9 +420,7 @@ class ClassModel(ObjectModel):
     @property
     def attr___mro__(self):
         if not self._instance.newstyle:
-            raise exceptions.AttributeInferenceError(
-                target=self._instance, attribute="__mro__"
-            )
+            raise AttributeInferenceError(target=self._instance, attribute="__mro__")
 
         mro = self._instance.mro()
         obj = node_classes.Tuple(parent=self._instance)
@@ -429,9 +430,7 @@ class ClassModel(ObjectModel):
     @property
     def attr_mro(self):
         if not self._instance.newstyle:
-            raise exceptions.AttributeInferenceError(
-                target=self._instance, attribute="mro"
-            )
+            raise AttributeInferenceError(target=self._instance, attribute="mro")
 
         # pylint: disable=import-outside-toplevel; circular import
         from astroid import bases
@@ -471,10 +470,11 @@ class ClassModel(ObjectModel):
         thus it might miss a couple of them.
         """
         # pylint: disable=import-outside-toplevel; circular import
-        from astroid import bases, scoped_nodes
+        from astroid import bases
+        from astroid.nodes import scoped_nodes
 
         if not self._instance.newstyle:
-            raise exceptions.AttributeInferenceError(
+            raise AttributeInferenceError(
                 target=self._instance, attribute="__subclasses__"
             )
 
@@ -555,7 +555,7 @@ class GeneratorModel(FunctionModel):
     def __new__(cls, *args, **kwargs):
         # Append the values from the GeneratorType unto this object.
         ret = super().__new__(cls, *args, **kwargs)
-        generator = astroid.MANAGER.builtins_module["generator"]
+        generator = AstroidManager().builtins_module["generator"]
         for name, values in generator.locals.items():
             method = values[0]
 
@@ -583,7 +583,7 @@ class AsyncGeneratorModel(GeneratorModel):
     def __new__(cls, *args, **kwargs):
         # Append the values from the AGeneratorType unto this object.
         ret = super().__new__(cls, *args, **kwargs)
-        astroid_builtins = astroid.MANAGER.builtins_module
+        astroid_builtins = AstroidManager().builtins_module
         generator = astroid_builtins.get("async_generator")
         if generator is None:
             # Make it backward compatible.
@@ -631,7 +631,7 @@ class ExceptionInstanceModel(InstanceModel):
 
     @property
     def attr___traceback__(self):
-        builtins_ast_module = astroid.MANAGER.builtins_module
+        builtins_ast_module = AstroidManager().builtins_module
         traceback_type = builtins_ast_module[types.TracebackType.__name__]
         return traceback_type.instantiate_class()
 
@@ -710,7 +710,7 @@ class DictModel(ObjectModel):
             def infer_call_result(self, caller, context=None):
                 yield obj
 
-        meth = next(self._instance._proxied.igetattr(name))
+        meth = next(self._instance._proxied.igetattr(name), None)
         return DictMethodBoundMethod(proxy=meth, bound=self._instance)
 
     @property
@@ -751,8 +751,8 @@ class PropertyModel(ObjectModel):
 
     # pylint: disable=import-outside-toplevel
     def _init_function(self, name):
-        from astroid.node_classes import Arguments
-        from astroid.scoped_nodes import FunctionDef
+        from astroid.nodes.node_classes import Arguments
+        from astroid.nodes.scoped_nodes import FunctionDef
 
         args = Arguments()
         args.postinit(
@@ -773,7 +773,7 @@ class PropertyModel(ObjectModel):
 
     @property
     def attr_fget(self):
-        from astroid.scoped_nodes import FunctionDef
+        from astroid.nodes.scoped_nodes import FunctionDef
 
         func = self._instance
 
@@ -781,7 +781,7 @@ class PropertyModel(ObjectModel):
             def infer_call_result(self, caller=None, context=None):
                 nonlocal func
                 if caller and len(caller.args) != 1:
-                    raise exceptions.InferenceError(
+                    raise InferenceError(
                         "fget() needs a single argument", target=self, context=context
                     )
 
@@ -795,7 +795,7 @@ class PropertyModel(ObjectModel):
 
     @property
     def attr_fset(self):
-        from astroid.scoped_nodes import FunctionDef
+        from astroid.nodes.scoped_nodes import FunctionDef
 
         func = self._instance
 
@@ -816,7 +816,7 @@ class PropertyModel(ObjectModel):
 
         func_setter = find_setter(func)
         if not func_setter:
-            raise exceptions.InferenceError(
+            raise InferenceError(
                 f"Unable to find the setter of property {func.function.name}"
             )
 
@@ -824,7 +824,7 @@ class PropertyModel(ObjectModel):
             def infer_call_result(self, caller=None, context=None):
                 nonlocal func_setter
                 if caller and len(caller.args) != 2:
-                    raise exceptions.InferenceError(
+                    raise InferenceError(
                         "fset() needs two arguments", target=self, context=context
                     )
                 yield from func_setter.infer_call_result(caller=caller, context=context)
