@@ -5,10 +5,11 @@
 # Copyright (c) 2018 hippo91 <guillaume.peillex@gmail.com>
 # Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
 # Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
+# Copyright (c) 2021 Alphadelta14 <alpha@alphaservcomputing.solutions>
 # Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
 
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/master/LICENSE
+# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
 
 
 """
@@ -21,8 +22,7 @@ leads to an inferred FrozenSet:
 """
 
 
-from astroid import bases, decorators, node_classes, scoped_nodes, util
-from astroid.const import BUILTINS
+from astroid import bases, decorators, util
 from astroid.exceptions import (
     AttributeInferenceError,
     InferenceError,
@@ -30,15 +30,16 @@ from astroid.exceptions import (
     SuperError,
 )
 from astroid.manager import AstroidManager
+from astroid.nodes import node_classes, scoped_nodes
 
 objectmodel = util.lazy_import("interpreter.objectmodel")
 
 
-class FrozenSet(node_classes._BaseContainer):
+class FrozenSet(node_classes.BaseContainer):
     """class representing a FrozenSet composite node"""
 
     def pytype(self):
-        return "%s.frozenset" % BUILTINS
+        return "builtins.frozenset"
 
     def _infer(self, context=None):
         yield self
@@ -118,7 +119,7 @@ class Super(node_classes.NodeNG):
         return ast_builtins.getattr("super")[0]
 
     def pytype(self):
-        return "%s.super" % BUILTINS
+        return "builtins.super"
 
     def display_type(self):
         return "Super of"
@@ -260,7 +261,10 @@ class PartialFunction(scoped_nodes.FunctionDef):
     def __init__(
         self, call, name=None, doc=None, lineno=None, col_offset=None, parent=None
     ):
-        super().__init__(name, doc, lineno, col_offset, parent)
+        super().__init__(name, doc, lineno, col_offset, parent=None)
+        # A typical FunctionDef automatically adds its name to the parent scope,
+        # but a partial should not, so defer setting parent until after init
+        self.parent = parent
         self.filled_positionals = len(call.positional_arguments[1:])
         self.filled_args = call.positional_arguments[1:]
         self.filled_keywords = call.keyword_arguments
@@ -302,7 +306,7 @@ class Property(scoped_nodes.FunctionDef):
     type = "property"
 
     def pytype(self):
-        return "%s.property" % BUILTINS
+        return "builtins.property"
 
     def infer_call_result(self, caller=None, context=None):
         raise InferenceError("Properties are not callable")
